local VERSION = tonumber(("$Revision: 44367 $"):match("%d+"))

local Parrot = Parrot
local Parrot_AnimationStyles = Parrot:NewModule("AnimationStyles")
if Parrot.revision < VERSION then
	Parrot.version = "r" .. VERSION
	Parrot.revision = VERSION
	Parrot.date = ("$Date: 2007-07-20 18:25:53 -0400 (Fri, 20 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local L = Parrot:L("Parrot_AnimationStyles")

local animationStyles = {}
local animationStylesCleanup = {}
local animationStylesValidate = {}
local animationStylesOverlap = {}

function Parrot_AnimationStyles:RegisterAnimationStyle(name, localName, func, cleanup, overlap)
	self = Parrot_AnimationStyles -- so people can Parrot:RegisterAnimationStyle
	AceLibrary.argCheck(self, name, 2, "string")
	AceLibrary.argCheck(self, localName, 3, "string")
	AceLibrary.argCheck(self, func, 4, "function")
	AceLibrary.argCheck(self, cleanup, 5, "function", "nil")
	AceLibrary.argCheck(self, overlap, 6, "boolean", "nil")
	if animationStyles[name] then
		error(("Animation style %q already registered"):format(name), 2)
	end
	animationStyles[name] = func
	animationStylesValidate[name] = localName
	animationStylesCleanup[name] = cleanup
	animationStylesOverlap[name] = overlap or false
end
Parrot.RegisterAnimationStyle = Parrot_AnimationStyles.RegisterAnimationStyle

function Parrot_AnimationStyles:HasAnimationStyle(name)
	self = Parrot_AnimationStyles -- so people can Parrot:RegisterAnimationStyle
	AceLibrary.argCheck(self, name, 2, "string")
	return not not animationStyles[name]
end
Parrot.HasAnimationStyle = Parrot_AnimationStyles.HasAnimationStyle

function Parrot_AnimationStyles:GetAnimationStyle(name)
	local func = animationStyles[name]
	if func then
		return func, animationStylesCleanup[name], animationStylesOverlap[name]
	else
		return animationStyles.Straight, animationStylesCleanup.Straight, animationStylesOverlap.Straight
	end
end

function Parrot_AnimationStyles:GetAnimationStylesValidate(name)
	return animationStylesValidate
end

------------------------------------------------------------------------------
-- Animation style registry --------------------------------------------------
------------------------------------------------------------------------------

Parrot_AnimationStyles:RegisterAnimationStyle("Straight", L["Straight"], function(fontstring, scrollArea, percent, reverse)
	-- simple vertical
	local fsHeight = fontstring:GetHeight()
	local height = scrollArea.size - fsHeight
	local middle = scrollArea.yOffset
	local bottom = middle - scrollArea.size/2
	local center = scrollArea.xOffset
	if reverse then
		percent = 1 - percent
	end
	local y = bottom + percent * height
	local x = center
	local align = scrollArea.align
	local point
	if align == "LEFT" then
		point = "BOTTOMLEFT"
	elseif align == "RIGHT" then
		point = "BOTTOMRIGHT"
	else -- center
		point = "BOTTOM"
	end
	fontstring:SetPoint(point, UIParent, "CENTER", x, y)
end, nil, true)
Parrot_AnimationStyles:RegisterAnimationStyle("Left parabola", L["Left parabola"], function(fontstring, scrollArea, percent, reverse)
	-- makes a parabola in the form of x = y^2 - 1
	local fsHeight = fontstring:GetHeight()
	local height = scrollArea.size - fsHeight
	local middle = scrollArea.yOffset
	local bottom = middle - scrollArea.size/2
	local center = scrollArea.xOffset
	if reverse then
		percent = 1 - percent
	end
	local y = bottom + percent * height
	local x = center
	local align = scrollArea.align
	local point
	if align == "LEFT" then
		point = "BOTTOMLEFT"
	elseif align == "RIGHT" then
		point = "BOTTOMRIGHT"
	else -- center
		point = "BOTTOM"
	end
	x = x - (1 - 4*(percent - 0.5)^2) * height * 1/3
	fontstring:SetPoint(point, UIParent, "CENTER", x, y)
end, nil, true)
Parrot_AnimationStyles:RegisterAnimationStyle("Right parabola", L["Right parabola"], function(fontstring, scrollArea, percent, reverse)
	-- makes a parabola in the form of x = -y^2 + 1
	local fsHeight = fontstring:GetHeight()
	local height = scrollArea.size - fsHeight
	local middle = scrollArea.yOffset
	local bottom = middle - scrollArea.size/2
	local center = scrollArea.xOffset
	if reverse then
		percent = 1 - percent
	end
	local y = bottom + percent * height
	local x = center
	local align = scrollArea.align
	local point
	if align == "LEFT" then
		point = "BOTTOMLEFT"
	elseif align == "RIGHT" then
		point = "BOTTOMRIGHT"
	else -- center
		point = "BOTTOM"
	end
	x = x + (1 - 4*(percent - 0.5)^2) * height * 1/3
	fontstring:SetPoint(point, UIParent, "CENTER", x, y)
end, nil, true)

Parrot_AnimationStyles:RegisterAnimationStyle("Left semicircle", L["Left semicircle"], function(fontstring, scrollArea, percent, reverse)
	-- makes a semicircle in the form of x = -(-y^2 + 1)^0.5
	local fsHeight = fontstring:GetHeight()
	local height = scrollArea.size - fsHeight
	local middle = scrollArea.yOffset
	local center = scrollArea.xOffset
	if reverse then
		percent = 1 - percent
	end
	local align = scrollArea.align
	local point
	if align == "LEFT" then
		point = "BOTTOMLEFT"
	elseif align == "RIGHT" then
		point = "BOTTOMRIGHT"
	else -- center
		point = "BOTTOM"
	end
	local y = middle - height/2 * sin(90 + 180 * (percent))
	local x = center + height/2 * cos(90 + 180 * (percent))
	fontstring:SetPoint(point, UIParent, "CENTER", x, y)
end, nil, true)
Parrot_AnimationStyles:RegisterAnimationStyle("Right semicircle", L["Right semicircle"], function(fontstring, scrollArea, percent, reverse)
	-- makes a semicircle in the form of x = (-y^2 + 1)^0.5
	local fsHeight = fontstring:GetHeight()
	local height = scrollArea.size - fsHeight
	local middle = scrollArea.yOffset
	local center = scrollArea.xOffset
	if reverse then
		percent = 1 - percent
	end
	local align = scrollArea.align
	local point
	if align == "LEFT" then
		point = "BOTTOMLEFT"
	elseif align == "RIGHT" then
		point = "BOTTOMRIGHT"
	else -- center
		point = "BOTTOM"
	end
	local y = middle - height/2 * sin(90 - 180 * (percent))
	local x = center + height/2 * cos(90 - 180 * (percent))
	fontstring:SetPoint(point, UIParent, "CENTER", x, y)
end, nil, true)

Parrot_AnimationStyles:RegisterAnimationStyle("Pow", L["Pow"], function(fontstring, scrollArea, percent, reverse, num, max)
	-- places in the center and shakes around
	local fsHeight = fontstring:GetHeight()
	local middle = scrollArea.yOffset
	local center = scrollArea.xOffset
	local topMiddle
	local topDiff = (max - 1) * 0.5 * fsHeight
	local currDiff = -(num - 1) * fsHeight
	local y
	if not reverse then
		y = middle + topDiff + currDiff
	else
		y = middle - topDiff - currDiff
	end
	
	local x = center
	local point = scrollArea.align
	-- now for squiggle
	local time = GetTime()
	if not fontstring.nextSquiggle or fontstring.nextSquiggle < time then
		fontstring.nextSquiggle = time + 0.05
		fontstring.squiggleX = math.random(-1, 1)
		fontstring.squiggleY = math.random(-1, 1)
	end	
	x = x + fontstring.squiggleX
	y = y + fontstring.squiggleY
	
	fontstring:SetPoint(point, UIParent, "CENTER", x, y)
end, function(fontstring, scrollArea)
	fontstring.nextSquiggle = nil
	fontstring.squiggleX = nil
	fontstring.squiggleY = nil
end, false)

Parrot_AnimationStyles:RegisterAnimationStyle("Static", L["Static"], function(fontstring, scrollArea, percent, reverse, num, max)
	-- places in the center
	local fsHeight = fontstring:GetHeight()
	local middle = scrollArea.yOffset
	local center = scrollArea.xOffset
	local topMiddle
	local topDiff = (max - 1) * 0.5 * fsHeight
	local currDiff = -(num - 1) * fsHeight
	local y
	if not reverse then
		y = middle + topDiff + currDiff
	else
		y = middle - topDiff - currDiff
	end
	
	local x = center
	local point = scrollArea.align
	
	fontstring:SetPoint(point, UIParent, "CENTER", x, y)
end, nil, false)

Parrot_AnimationStyles:RegisterAnimationStyle("Right rainbow", L["Right rainbow"], function(fontstring, scrollArea, percent, reverse, num, max, id)
	-- makes a parabola in the form of y = -(x - 1)(x + 0.5) + 1
	local fsHeight = fontstring:GetHeight()
	local height = scrollArea.size - fsHeight
	local middle = scrollArea.yOffset
	local center = scrollArea.xOffset
	local y = bottom + percent * height
	local x = center
	local align = scrollArea.align
	local point
	if align == "LEFT" then
		point = "BOTTOMLEFT"
	elseif align == "RIGHT" then
		point = "BOTTOMRIGHT"
	else -- center
		point = "BOTTOM"
	end
	x = center + height/2 * percent
	y = height * -16/9 * (percent + 1) * (percent - 0.5)
	if reverse then
		y = -y
	end
	y = middle + y
	fontstring:SetPoint(point, UIParent, "CENTER", x, y)
end, nil, true)

Parrot_AnimationStyles:RegisterAnimationStyle("Left rainbow", L["Left rainbow"], function(fontstring, scrollArea, percent, reverse, num, max, id)
	-- makes a parabola in the form of y = -(x + 1)(x - 0.5) + 1
	local fsHeight = fontstring:GetHeight()
	local height = scrollArea.size - fsHeight
	local middle = scrollArea.yOffset
	local center = scrollArea.xOffset
	local align = scrollArea.align
	local point
	if align == "LEFT" then
		point = "BOTTOMLEFT"
	elseif align == "RIGHT" then
		point = "BOTTOMRIGHT"
	else -- center
		point = "BOTTOM"
	end
	x = center - height/2 * percent
	y = height * -16/9 * (percent + 1) * (percent - 0.5)
	if reverse then
		y = -y
	end
	y = middle + y
	fontstring:SetPoint(point, UIParent, "CENTER", x, y)
end, nil, true)
